<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Setforms2\Shared\ORM\DateTime;

class Pla07600AprovacaoCardapioFilial extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "aprovacao_cardapio_filial_servicos";
    protected $parameterBagColumns = array('P_CDFILIAL', 'P_NRORG', 'CDOPERADOR','CDGRUPFILI');
    protected $novoCodigo;
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $elaboracaoService;
    protected $aprovacaoCardapioFilialService;

    public function __construct($crudService, $parameterBag, Manager $dataSourceManager, Environment $environment, $novoCodigo, $entityManager, $elaboracaoService, $aprovacaoCardapioFilialService) {

        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
    	$this->novoCodigo = $novoCodigo;
    	$this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->elaboracaoService = $elaboracaoService;
    	$this->aprovacaoCardapioFilialService = $aprovacaoCardapioFilialService;

    }

    public function retornaTiposDeCusto(DTO\Request\Filter $request, DTO\Response $response) {
    	$conditions = $request->getFilterCriteria()->getConditions();
    	$response->addDataset($this->findDataSet('aprovacao_cardapio_filial_tipo_de_custo_default', $conditions, array('P_CDFILIAL','P_STIDIOMA'), 'tipoDeCustoPadrao'));
    }

    public function findDataSet($dataSourceName, $conditions = array(), $parameterBagColumns = array(), $dataSetName = null) {

    	$dataSetName = isset($dataSetName) ? $dataSetName : $dataSourceName;
    	$this->parameterBagColumns = $parameterBagColumns;

    	$filterCriteria = $this->createFilterCriteria($dataSourceName, $conditions);
        $filterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$dataSet->setDataSourceName($dataSetName);

		return $dataSet;

    }

    public function createFilterCriteria($dataSourceName, $conditions = array()) {

    	$filterCriteria = new FilterCriteria($dataSourceName);

    	if (isset($conditions)) {

	    	foreach ($conditions as $condition) {

	    		if (isset($condition['operator'])) {

	    			$filterCriteria->addCondition($condition['columnName'], $condition['operator'], $condition['value']);

	    		} else {

	    			$filterCriteria->addCondition($condition['columnName'], $condition['value']);
	    		}
	    	}
    	}

    	return $filterCriteria;
    }
    
    public function verificaRestricao(DTO\Request $request, DTO\Response $response) {
        $sql = "SELECT DISTINCT CDFILIAL, CDSERVFILI
            FROM TABLE (RESTR_CONTRATUAIS_CARD(:P_CDFILIAL, :P_CDSERVFILI, NULL, :P_DTINICIAL, :P_DTFINAL))";
        
        $filter = $request->getParameters()['filter'];
        
        $params = array(
            "P_CDSERVFILI" => $filter[0]['value'],
            "P_CDFILIAL" => $filter[1]['value'],
            "P_DTINICIAL" => $filter[2]['value'],
            "P_DTFINAL" => $filter[3]['value']
        );
        
        $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('pla07600_restricao', $result));
        
    }

    public function retornaCardapiosAprovacao(DTO\Request\Row $request, DTO\Response $response) {

    	$row = $request->getRow();
        $nrlista = $row['NRLISTASERVICO'];
    	$parameterBagColumns = array('P_NRLISTA', 'P_DTMESCARD');
    	$conditions = array(
    		array('columnName' => 'P_NRLISTA', 'operator' => '=', 'value' => $nrlista,),
    		array('columnName' => 'P_DTMESCARD', 'operator' => '=', 'value' => $row['DTMESCARD'],),
    	);


    	$dataSet = $this->findDataSet('aprovacao_cardapio_filial_cardapios', $conditions, $parameterBagColumns, 'cardapiosAprovacao');

        $tipoDeOperador = $this->elaboracaoService->retornaTipoOperador();

        $response->addDataset($dataSet);
    	$response->addDataset(new DataSet('TIPODEOPERADOR', array($tipoDeOperador)));

    }

    public function insereLista($nrlista, $row, $cdfilial) {

    	$arrayCdlista = $row['CDSERVICO'];

    	if ($row['checkboxServico'] == 'T') {

    		$arrayCdlista = $this->retornaCdlista($cdfilial);
    	}

    	foreach ($arrayCdlista as $cdlista) {

	    	$this->insertLista($cdlista, $nrlista);

    	}

    }

    public function insertLista($cdlista, $nrlista) {

        $lista = new \Geral\Entities\Lista();
        $lista = $this->buildLista($lista, $nrlista, $cdlista);

        $this->entityManager->persist($lista);
        $this->entityManager->flush();

    }

    public function removeLista($nrlista) {

    	$params = array(
            'nrlista' => $nrlista,
            'nrorg' => $this->environment->getNrOrg(),
        );

        $listas = $this->entityManager->getRepository('Geral\Entities\Lista')->findBy($params);

        foreach ($listas as $lista) {

            $this->entityManager->remove($lista);
            $this->entityManager->flush();
        }

    }

    public function retornaCdlista($cdfilial) {

    	$cdlista = array();
    	$conditions = array(
    		array('columnName' => 'P_CDFILIAL', 'operator' => '=', 'value' => $cdfilial,),
    		array('columnName' => 'P_CDSERVFILI', 'operator' => '=', 'value' => 'T',),
    		array('columnName' => 'CDOPERADOR', 'operator' => '=', 'value' => $this->environment->getCdOperador(),),
    		array('columnName' => 'NRORG', 'operator' => '=', 'value' => $this->environment->getNrOrg(),),
    	);
    	$dataSet = $this->findDataSet('aprovacao_cardapio_filial_cdlista', $conditions, array('P_CDFILIAL', 'P_CDSERVFILI', 'CDOPERADOR', 'NRORG'));
    	foreach ($dataSet->getRows() as $row) {
    		$cdlista[] = $row['CDLISTA'];
    	}
    	return $cdlista;
    }

    public function buildLista($lista, $nrlista, $cdlista) {

        $lista->setNrlista($nrlista);
        $lista->setCdlista($cdlista);
        $lista->setDtultatulist(new DateTime());

        $lista->setNrorg($this->environment->getNrOrg());
        $lista->setDtinclusao(new DateTime());
        $lista->setDtultatu(new DateTime());
        $lista->setCdoperinclusao($this->environment->getCdOperador());
        $lista->setIdativo('S');

        return $lista;
    }

    public function verificaCusto(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();

        $this->salvaObservacaoGeral($row['OBSERVACOES_GERAIS'], $row['DATAINICIAL']);

        $filiaisVerificarCusto = $this->retornaFiliaisVerificarCusto($row['FILIAIS_APROVACAO']);

        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrOrg());

        $cardapiosParaAprovar = $this->retornaCardapiosParaAprovar($nrlista, $row);

        $cardapiosValidados = $this->validaCardapiosParaAprovar($cardapiosParaAprovar, $row, $filiaisVerificarCusto);

        if (count($cardapiosParaAprovar) > 0) {

            $cardapios = $this->verificaCustoDosCardapios($cardapiosValidados);

            if (count($cardapios['FORA_DA_META']) > 0) {

                $response->addDataset(new DataSet('cardapios', $cardapios));

            } else {

                $this->aprovarCardapios($cardapios['DENTRO_DA_META'], $row['CARDAPIOS_SELECIONADOS']);
            }

        } else {

            $this->aprovarCardapios($cardapiosValidados, $row['CARDAPIOS_SELECIONADOS']);
        }

    }

    public function validaCardapiosParaAprovar($cardapiosParaAprovar, $row, $filiaisVerificarCusto) {

        $cardapiosSelecionados = $row['CARDAPIOS_SELECIONADOS'];
        $cardapios = array();

        foreach ($cardapiosParaAprovar as $cardapioParaAprovar) {

            $cardapios[] = $this->criaCardapioValido($cardapioParaAprovar, $filiaisVerificarCusto, $row);

        }

        if (count($cardapiosParaAprovar) != count($cardapiosSelecionados)) {

            foreach ($cardapiosSelecionados as $cardapioSelecionado) {

                if ($this->naoExisteNoArrayCardapio($cardapiosParaAprovar, $cardapioSelecionado)) {

                    $cardapios[] = $this->criaCardapioValido($cardapioSelecionado, $filiaisVerificarCusto, $row);
                }
            }

        }

        return $cardapios;
    }

    public function naoExisteNoArrayCardapio($cardapiosParaAprovar, $cardapioSelecionado) {

        $naoExiste = true;

        foreach ($cardapiosParaAprovar as $cardapioParaAprovar) {

            if ($cardapioParaAprovar['CDFILIAL'] == $cardapioSelecionado['CDFILIAL'] &&
                $cardapioParaAprovar['CDSERVFILI'] == $cardapioSelecionado['CDSERVFILI']) {

                $naoExiste = false;
                break;
            }
        }

        return $naoExiste;
    }

    public function criaCardapioValido($cardapioSelecionado, $filiaisVerificarCusto, $row) {

        return array(
            "CDFILIAL" => $cardapioSelecionado['CDFILIAL'],
            "CDSERVFILI" => $cardapioSelecionado['CDSERVFILI'],
            "VRPEMAXDIFPLAPA" => $filiaisVerificarCusto[$cardapioSelecionado['CDFILIAL']]['VRPEMAXDIFPLAPA'],
            "VRPEMINDIFPLAPA" => $filiaisVerificarCusto[$cardapioSelecionado['CDFILIAL']]['VRPEMINDIFPLAPA'],
            "CDTIPOCUSTODESC" => $row['TIPODECUSTO'],
            "DTMESCARD" =>  $row['DATAINICIAL'],
            "VRCUSTPRODMAT" => isset($cardapioSelecionado['VRCUSTPRODMAT']) ? $cardapioSelecionado['VRCUSTPRODMAT'] : null,
            "VRCUSTPADR" => isset($cardapioSelecionado['VRCUSTPADR']) ? $cardapioSelecionado['VRCUSTPADR'] : null,
            "DIFECUSTO" => isset($cardapioSelecionado['DIFECUSTO']) ? $cardapioSelecionado['DIFECUSTO'] : null,
        );

    }

    public function retornaFiliaisVerificarCusto($filiaisAprovacao) {

        $filiaisVerificarCusto = array();

        foreach ($filiaisAprovacao as $filialAprovacao) {

            $paramPlaneja = $this->elaboracaoService->retornaParamplaneja($filialAprovacao);

            $vrpemindifplapa = floatval(str_replace(',', '.', $paramPlaneja['VRPEMINDIFPLAPA']));
            $vrpemaxdifplapa = floatval(str_replace(',', '.', $paramPlaneja['VRPEMAXDIFPLAPA']));

            if ($vrpemindifplapa > 0 || $vrpemaxdifplapa > 0) {

                $filiaisVerificarCusto[$filialAprovacao] = array(
                    'VRPEMINDIFPLAPA' => $vrpemindifplapa,
                    'VRPEMAXDIFPLAPA' => $vrpemaxdifplapa
                );

            } else {

                $filiaisVerificarCusto[$filialAprovacao] = array(
                    'VRPEMINDIFPLAPA' => 0,
                    'VRPEMAXDIFPLAPA' => 0
                );

            }

        }

        return $filiaisVerificarCusto;
    }

    public function retornaCardapiosParaAprovar($nrlista, $row) {

        foreach ($row['CDSERVICO'] as $cdlista) {

            $this->insertLista($cdlista, $nrlista);

        }

        $procedure = $this->executaConsplanGeral($row, $nrlista);

        $this->removeLista($nrlista);

        $dataSet = $this->retornaDataSetCustos($row, $procedure['P_NRCONSPLAN']);

        return $dataSet->getRows();

    }

    public function executaConsplanGeral($row, $nrlista) {

        $consplanGeral = new StoredProcedure($this->entityManager->getConnection(), 'CONSPLAN_GERAL');

        $consplanGeral->addParam(new Param('P_NRLISTA',      Param::PARAM_INPUT, $nrlista,            Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DATAINICIAL'], Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DATAFINAL'],   Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, 'CFP',               Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'P',                 Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                   Param::PARAM_TYPE_INT));
        $consplanGeral->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, 'N',                 Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, '0',                 Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, 'zzzzzzzzzz',        Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],  Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_NRCONSPLANTH', Param::PARAM_INPUT, 'GERANOVCOD',        Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_OUTPUT, null,               Param::PARAM_TYPE_STR, 20));

        return $consplanGeral->execute();
    }

    public function verificaCustoDosCardapios($cardapios) {

        $cardapiosPorMeta = array(
            'FORA_DA_META' => array(),
            'DENTRO_DA_META' => array(),
        );

        foreach ($cardapios as $cardapio) {

            $cardapio['DIFECUSTO'] = isset($cardapio['DIFECUSTO']) ? str_replace(',', '.', $cardapio['DIFECUSTO']) : 0;

            if (floatval($cardapio['VRPEMAXDIFPLAPA']) == 0 && floatval($cardapio['VRPEMINDIFPLAPA']) == 0) {
                $cardapiosPorMeta['DENTRO_DA_META'][] = $cardapio;
            } else if ($cardapio['DIFECUSTO'] > floatval($cardapio['VRPEMAXDIFPLAPA'])){
                $cardapiosPorMeta['FORA_DA_META'][] = $cardapio;
            } else if ($cardapio['DIFECUSTO'] < 0 && abs($cardapio['DIFECUSTO']) > floatval($cardapio['VRPEMINDIFPLAPA'])){
                $cardapiosPorMeta['FORA_DA_META'][] = $cardapio;
            } else {
                $cardapiosPorMeta['DENTRO_DA_META'][] = $cardapio;
            }

        }

        return $cardapiosPorMeta;

    }

    public function retornaDataSetCustos($row, $nrconsplan) {

        $parameterBagColumns = array('P_CDTIPOCUSTO', 'P_DTINICIAL', 'P_DTFINAL', 'P_NRCONSPLAN');

        $conditions = array(
            array('columnName' => 'P_CDTIPOCUSTO',  'operator' => '=', 'value' => $row['TIPODECUSTO'],),
            array('columnName' => 'P_DTINICIAL',    'operator' => '=', 'value' => $row['DATAINICIAL'],),
            array('columnName' => 'P_DTFINAL',      'operator' => '=', 'value' => $row['DATAFINAL'],),
            array('columnName' => 'P_NRCONSPLAN',   'operator' => '=', 'value' => $nrconsplan,),
        );

        return $this->findDataSet('aprovacao_cardapio_filial_custos', $conditions, $parameterBagColumns, 'custos');
    }

    public function aprovarCardapios($cardapios, $cardapiosSelecionados) {

        foreach ($cardapios as $cardapio) {

            $cardapioSelecionado = $this->retornaCardapioSelecionado($cardapiosSelecionados, $cardapio);
            $cardapio['DTMESCARD'] = $this->elaboracaoService->dateToDateTime(explode(' ', $cardapio['DTMESCARD'])[0]);
            $cardapio['IDSITUOPERCARD'] = 'S';

            $this->aprovacaoCardapioFilialService->updateCtrlaprovcard($cardapio, $cardapioSelecionado, new DateTime());
            $this->aprovacaoCardapioFilialService->insertLogaprovcard($cardapio);
        }

    }

    public function retornaCardapioSelecionado($cardapiosSelecionados, $cardapio) {

        $cardSelecionado = array();

        foreach ($cardapiosSelecionados as $cardapioSelecionado) {

            if ($cardapioSelecionado['CDFILIAL'] == $cardapio['CDFILIAL'] &&
                $cardapioSelecionado['CDSERVFILI'] == $cardapio['CDSERVFILI']) {

                $cardSelecionado = $cardapioSelecionado;
                break;

            }
        }

        return $cardSelecionado;
    }

    public function cancelarAprovacao(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();

        $this->salvaObservacaoGeral($row['OBSERVACOES_GERAIS'], $row['DATAINICIAL']);

        foreach ($row['CARDAPIOS_SELECIONADOS'] as $cardapio) {

            $cardapio['DTMESCARD'] = $this->elaboracaoService->dateToDateTime($row['DATAINICIAL']);
            $cardapio['VRCUSTPRODMAT'] = null;
            $cardapio['CDTIPOCUSTODESC'] = null;
            $cardapio['IDSITUOPERCARD'] = 'N';

            $this->aprovacaoCardapioFilialService->updateCtrlaprovcard($cardapio, $cardapio, null);
            $this->aprovacaoCardapioFilialService->insertLogaprovcard($cardapio);
        }

    }

    public function verificaOperadorTemPermissao(DTO\Request $request, DTO\Response $response) {

        $temPermissao = $this->aprovacaoCardapioFilialService->operadorTemPermissaoAprovarForaDaMeta();

        $response->addDataset(new DataSet('permissao', array($temPermissao)));

    }

    public function aprovarCardapioForaDaMeta(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();

        $filiaisVerificarCusto = $this->retornaFiliaisVerificarCusto($row['FILIAIS_APROVACAO']);

        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrOrg());

        $cardapiosParaAprovar = $this->retornaCardapiosParaAprovar($nrlista, $row);

        $cardapiosValidados = $this->validaCardapiosParaAprovar($cardapiosParaAprovar, $row, $filiaisVerificarCusto);

        $this->aprovarCardapios($cardapiosValidados, $row['CARDAPIOS_SELECIONADOS']);
    }

    public function salvaObservacaoGeral($observacoesGerais, $dataInicial) {

        $dataInicial = DateTime::createFromFormat('!d/m/Y', $dataInicial);

        foreach ($observacoesGerais as $filial => $observacao) {

            if (empty($observacao)) {

                $this->aprovacaoCardapioFilialService->removeObsaprovcardfili($filial, $dataInicial);

            } else {

                $this->aprovacaoCardapioFilialService->insertObsaprovcardfili($filial, $observacao, $dataInicial);
            }
        }
    }

    public function salvarObservacaoGeral(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();
        $obs = $row['OBSERVACOES_GERAIS'];
        $dataInicial = $row['DTMESCARD'];

        $this->salvaObservacaoGeral($obs, $dataInicial);
    }
}