<?php

namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\FilterCriteria;

class VerificacaoPlanejamentoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $novoCodigo;
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService       = $crudService;
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
    }
    
    /**
     * Rotina de validação para liberação ou bloqueio de telas do planejamento.
     * 
     * @param {DTO\Request} $request Requisição vinda
     *  do usuário.
     * @param {DTO\Reponse} $response Instância de 
     *  resposta a requisição do usuário.
     */
    
    public function validacaoPlanejamento(DTO\Request $request, DTO\Response $response) {
        
        $connection = $this->entityManager->getConnection();
        
        if(!($this->verificaOperadorModoMatriz($connection))){
            
            $servicosUtCardDesc = $this->servicosUtCardapioDescen($connection);
            if(sizeof($servicosUtCardDesc) > 0){
                
                $servicos = $this->servicosNaoAprovados($connection);
                
                $mensagem = $this->mensagemBloqueio($connection);

                $dataSet = array(
                    'servicos' => $servicos,
                    'mensagem' => $mensagem
                );
                $response->addDataSet(new DataSet('validacaoPlanejamento', $dataSet));
            }
        } else {
            $dataSet = array(
                'servicos' => array(), 
                'mensagem' => false
            );
            
            $response->addDataSet(new DataSet('validacaoPlanejamento', $dataSet));
        }
        
    }
    
    /**
     * Verifica se o operador está em modo matriz.
     *  
     */
    
    private function verificaOperadorModoMatriz($connection) {
        /* Verifica se perfil de acesso com permissão de matriz está parametrizado */
        $nrperfilacesso = $this->perfilAcessoMatriz($connection);
        $result = false;
        if (!empty($nrperfilacesso)) {
            $sql = "SELECT O.CDOPERADOR
                    FROM OPERADOR O
                    INNER JOIN PERFACESSPARC P ON P.NRPARCNEGOCIO = O.NRPARCNEGOCIO
                    WHERE O.CDOPERADOR = :P_CDOPERADOR
                    AND P.NRPERFILACESSO = :P_NRPERFILACESSO";
            $query = $connection->prepare($sql);
            $params = array(
                "P_CDOPERADOR" => $this->environment->getCdOperador(),
                "P_NRPERFILACESSO" => $nrperfilacesso
            );
            $query->execute($params);
            $result = $query->fetch();
            if($result == false)
                return false;
            return true;
        }
        
        $cdgrupoper = $this->grupoMatriz($connection);
        
        if(!empty($cdgrupoper["CDGRUPOPER"])) {
            $sql = "SELECT CDGRUPOPER,
                           CDOPERADOR
                      FROM OPERGRUPOP
                     WHERE CDOPERADOR = :P_CDOPERADOR
                       AND CDGRUPOPER = :P_CDGRUPOPER
                       AND NRORG = :P_NRORG";
                       
    	    $query = $connection->prepare($sql);
    	    
            $params = array(
                "P_CDOPERADOR" => $this->environment->getCdOperador(),
                "P_CDGRUPOPER" => $cdgrupoper["CDGRUPOPER"],
                "P_NRORG"      => $this->environment->getNrOrgTrab()
            );
            
            $query->execute($params);
            
            $result = $query->fetchAll(\PDO::FETCH_ASSOC);
            return sizeof($result) > 0;
        } else {
            return true;
        }
        
    }
    
    private function grupoMatriz($connection) {
        $sql = "SELECT CDGRUPOPER
                  FROM AUTGRUPOP
                 WHERE IDAUTGRUPOP = 'LRCD' 
                   AND NRAUTGRUPOP = '000'
                   AND NRORG = :P_NRORG";
                   
	    $query = $connection->prepare($sql);
	    
        $params = array(
            "P_NRORG" => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        return $result;
    }
    
    /**
     * Retorna grupo que representa a matriz.
     *  
     */
    
    /**
     * Retorna serviços que utilizam cardápio descentralizado e caso a parâmetros
     * gerais esteja definida como cardápio descentralizado.
     * 
     */
    
    private function servicosUtCardapioDescen($connection) {
        
        $sql = "SELECT SERVICO.CDSERVFILI 
                  FROM SERVICO, 
                       PARAMGERAL 
                 WHERE SERVICO.CDFILIAL        = :P_CDFILIAL
                   AND SERVICO.NRORG           = :P_NRORG
                   AND SERVICO.IDCARDDESCENTR  = 'S'
                   AND PARAMGERAL.NRORG        = :P_NRORG
                   AND PARAMGERAL.CDPARGERAL   = '0001'
                   AND PARAMGERAL.IDUTCARDPADR = 'D'";
                   
	    $query = $connection->prepare($sql);
	    
        $params = array(
            "P_CDFILIAL" => $this->environment->getCdfilial(),
            "P_NRORG" => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        return $result;
    }
    
    /**
     * Retorna serviços não aprovados
     * 
     */
    
    private function servicosNaoAprovados($connection) {
        
        $sql = "SELECT CT.CDSERVFILI, 
                       SE.NMSERVICO, 
                       CT.CDSERVFILI|| ' ' ||SE.NMSERVICO AS DSSERVICO,
                       TO_CHAR(CA.DTLIMAPROVFILI + NVL(QTDIAPRORAPROV, 0), 'DD/MM/YYYY') AS DTLIMAPROVFILI
                  FROM CTRLAPROVCARD CT, 
                       CALENCARD CA, 
                       SERVICO SE
                 WHERE CT.DTAPROCARDFILI IS NULL
                   AND CA.DTLIMAPROVFILI IS NOT NULL
                   AND CT.CDFILIAL = :P_CDFILIAL
                   AND CT.DTMESCARDAPROV >= :P_DTMESCARD
                   AND CT.NRORG = :P_NRORG
                   AND CT.CDFILIAL = CA.CDFILIAL
                   AND CT.DTMESCARDAPROV = CA.DTMESCALENCARD
                   AND SE.IDCARDDESCENTR = 'S'
                   AND SE.CDFILIAL = CT.CDFILIAL
                   AND SE.CDSERVFILI = CT.CDSERVFILI";
                   
	    $query = $connection->prepare($sql);
	    
        $params = array(
            "P_CDFILIAL" => $this->environment->getCdfilial(),
            "P_DTMESCARD" => $this->environment->getCurrentTime()->modify('first day of this month')->format("d/m/Y"),
            "P_NRORG" => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        return $result;
        
    }
    
    /**
     * Retorna a mensagem de bloqueio disponível
     * 
     */
    
    private function mensagemBloqueio($connection) {
        
        $sql = "SELECT TO_CHAR(:P_QTDIAS) AS DIAS, 
                       DSMENSAPROVCARD
                  FROM MENSAPROVCARD
                 WHERE QTDIAMENS = :P_QTDIAS";
                   
	    $query = $connection->prepare($sql);
	    
        $params = array(
            "P_QTDIAS" => $this->diasParaDataLimite($connection)
        );
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        return $result;
        
    }
    
    /**
     * Retorna a quantidade de dias para o limite da aprovação
     * 
     */
    
    private function diasParaDataLimite($connection) {
        
        $sql = "SELECT CALEN.DTMESCALENCARD,
                       CALEN.DTLIMAPROVFILI + NVL(CALEN.QTDIAPRORAPROV, 0) AS DATALIMITE,
                       (CALEN.DTLIMAPROVFILI + NVL(CALEN.QTDIAPRORAPROV, 0)) - TO_DATE(:P_DTATUAL, 'DD/MM/YYYY') AS QTDIAS
                  FROM CALENCARD CALEN
                 WHERE CALEN.CDFILIAL = :P_CDFILIAL
                   AND CALEN.DTMESCALENCARD >= :P_DTMESCARD
                   AND CALEN.NRORG >= :P_NROG
                   AND CALEN.DTPREVLIBECARD IS NOT NULL
                   AND CALEN.DTLIMAPROVFILI IS NOT NULL
                   AND CALEN.DTLIMAPROVMATR IS NOT NULL
                   AND EXISTS ( SELECT CTRL.DTMESCARDAPROV
                                  FROM CTRLAPROVCARD CTRL, SERVICO SE
                                 WHERE CTRL.CDFILIAL = :P_CDFILIAL
                                   AND CTRL.DTAPROCARDFILI IS NULL
                                   AND CALEN.DTMESCALENCARD = CTRL.DTMESCARDAPROV
                                   AND SE.IDCARDDESCENTR = 'S'
                                   AND SE.CDFILIAL = CTRL.CDFILIAL
                                   AND SE.CDSERVFILI = CTRL.CDSERVFILI )
                 ORDER BY CALEN.DTMESCALENCARD";
                   
	    $query = $connection->prepare($sql);
	    
        $params = array(
            "P_DTATUAL" => $this->environment->getCurrentTime()->format("d/m/Y"),
            "P_CDFILIAL" => $this->environment->getCdfilial(),
            "P_DTMESCARD" => $this->environment->getCurrentTime()->modify('first day of this month')->format("d/m/Y"),
            "P_NROG" => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        if(!$result)
           return -1;
        return $result['QTDIAS'] >= 0? $result['QTDIAS']: -1;
    }
    
    private function perfilAcessoMatriz($connection){
        $sql = "SELECT NRPERFILACESSO FROM AUTPERFILACESSO A WHERE IDAUTPERFACESSO = 'LRCD'";
        $query = $connection->prepare($sql);
        $query->execute();
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        if ($result != false) {
            return $result["NRPERFILACESSO"];
        }
        return "";
    }
    
}