<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;
/**
 * 01/06/2015
 * @author Francis
 */
 
use \Zeedhi\Framework\DataSource\FilterCriteria; 

class ImportarCardapio {

    const SOBREPOR = 'S';
    const INCLUIDO_MATRIZ = 'N';
    const ALTERADO_FILIAL = 'S';
    const ALTERADO_MATRIZ = 'R';
    const PARAMGERAL = 'paramgeral';

	protected $entityManager;
    protected $novoCodigo;
    /** @var Environment */
    protected $environment;
    protected $nrcardapio;
    protected $dataSourceManager;
    protected $auth;

    public function __construct($entityManager, $environment, $novoCodigo, $dataSourceManager, $auth) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->novoCodigo = $novoCodigo;
        $this->dataSourceManager = $dataSourceManager;
        $this->auth = $auth;
    }
    
    public function retornaParamgeral() {
        
        $filterCriteria = new FilterCriteria(self::PARAMGERAL);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        
        return $rows[0];
    }

    private function retornaObjetoData($stringData) {

        $array = explode("/", $stringData);
        return new \DateTime($array[2]."-". $array[1]."-". $array[0]);

    }

    private function inserirComposicao($idpratoespe, $nrordcnfmes, $cdtipoprat, $nrseqcnfmes, $cdfilauxprat, $cdprato, $vrpeconscard, $filialDestino) {

        $dataAtual = new \DateTime();

        $compcard = new \Teknisa\Model\Planejamento\Entities\Compcard;

        $compcard->setCdfilial($filialDestino);
        $compcard->setNrCardapio($this->nrcardapio);
        $compcard->setNrseqpratcar($nrseqcnfmes);
        $compcard->setNrordpratcar($nrordcnfmes);
        $compcard->setIdpratoespe($idpratoespe);
        $compcard->setCdtipoprat($cdtipoprat);
        $compcard->setCdfilauxprat($cdfilauxprat);
        $compcard->setCdprato($cdprato);
        $compcard->setVrpeconscard($vrpeconscard);
        $compcard->setNrorg($this->environment->getNrOrg());
        $compcard->setDtinclusao($dataAtual);
        $compcard->setDtultatu($dataAtual);
        $compcard->setCdoperinclusao($this->environment->getCdOperador());

        $this->entityManager->persist($compcard);
        $this->entityManager->flush();

    }

    private function retornaComposicoes($nrcardapio, $cdfilial) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $nrcardapio,
                    "cdfilial" => $cdfilial
                ),
                array('nrordpratcar' => 'ASC')
            );
    }

    private function retornaConfiguracaoDoCardapio($cdservfili, $dtcardapio, $filialDestino) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cnfcames')
            ->findBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "cdfilial" => $filialDestino,
                    "dtrefecnfmes" => $this->retornaObjetoData($dtcardapio)->modify('first day of'),
                    "cdservfili" => $cdservfili
                ),
                array('nrordcnfmes' => 'ASC')
            );
    }

    private function retornaComposicoesFormatadas($nrcardapio, $cdfilial, $cdservfili, $dtcardapio, $filialDestino) {

        $configuracoes = $this->retornaConfiguracaoDoCardapio($cdservfili, $dtcardapio, $filialDestino);

        $composicoes = $this->retornaComposicoes($nrcardapio, $cdfilial);

        $composicaoFormatada = array();

        foreach ($configuracoes as $configuracao) {

            foreach ($composicoes as $key => $value) {

                if($value->getCdtipoprat() == $configuracao->getCdtipoprat()) {

                    $composicaoFormatada[] = array(
                        'nrseqcnfmes' => $configuracao->getNrseqcnfmes(),
                        'cdtipoprat' => $value->getCdtipoprat(),
                        'cdprato' => $value->getCdprato(),
                        'cdfilauxprat' => $value->getCdfilauxprat(),
                        'vrpeconscard' => $value->getVrpeconscard(),
                        'nrordcnfmes' => $configuracao->getNrordcnfmes()
                    );

                    unset($composicoes[$key]);

                    break;

                }

            }

        }

        return $composicaoFormatada;

    }

    private function inserirCardapio($comensais, $clientes, $funcionarios, $doacao, $dtcardapio, $cdservfili, $filialDestino) {

        $dataAtual = new \DateTime();

        $nrcardapio = $this->novoCodigo->getCodeByTableName('CARDAPIO'.$filialDestino, $this->environment->getNrOrg());

        $cardadia = new \Teknisa\Model\Planejamento\Entities\Cardadia;

        $cardadia->setCdfilial($filialDestino);
        $cardadia->setNrCardapio($nrcardapio);
        $cardadia->setCdservfili($cdservfili);
        $cardadia->setDtcardapio($this->retornaObjetoData($dtcardapio));
        $cardadia->setDtmescard($this->retornaObjetoData($dtcardapio)->modify('first day of'));
        $cardadia->setCdoperador($this->environment->getCdOperador());
        $cardadia->setQtcliesticar($clientes);
        $cardadia->setQtfunesticar($funcionarios);
        $cardadia->setQtdoaesticar($doacao);
        $cardadia->setQtcomesticar($comensais);
        $cardadia->setNrorg($this->environment->getNrOrg());
        $cardadia->setDtinclusao($dataAtual);
        $cardadia->setDtultatu($dataAtual);
        $cardadia->setCdoperinclusao($this->environment->getCdOperador());

        $this->entityManager->persist($cardadia);
        $this->entityManager->flush();

        $this->nrcardapio = $nrcardapio;

    }

    private function retornaEstimados($cdservfili, $diaSemana, $cdfilial) {

        $servico = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Servico')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "cdfilial" => $cdfilial,
                    "cdservfili" => $cdservfili
                )
            );

        if($servico) {

            $estimados = array();

            switch ($diaSemana) {

                case "DOM":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestidom(),
                        "QTFUNESTICAR" => $servico->getQtfunestidom(),
                        "QTDOAESTICAR" => $servico->getQtdoaestidom(),
                        "QTCOMESTICAR" => $servico->getQtcomestidom()
                    );

                    break;

                case "SEG":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiseg(),
                        "QTFUNESTICAR" => $servico->getQtfunestiseg(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiseg(),
                        "QTCOMESTICAR" => $servico->getQtcomestiseg()
                    );

                    break;

                case "TER":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiter(),
                        "QTFUNESTICAR" => $servico->getQtfunestiter(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiter(),
                        "QTCOMESTICAR" => $servico->getQtcomestiter()
                    );

                    break;

                case "QUA":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiqua(),
                        "QTFUNESTICAR" => $servico->getQtfunestiqua(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiqua(),
                        "QTCOMESTICAR" => $servico->getQtcomestiqua()
                    );

                    break;

                case "QUI":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiqui(),
                        "QTFUNESTICAR" => $servico->getQtfunestiqui(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiqui(),
                        "QTCOMESTICAR" => $servico->getQtcomestiqui()
                    );

                    break;

                case "SEX":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestisex(),
                        "QTFUNESTICAR" => $servico->getQtfunestisex(),
                        "QTDOAESTICAR" => $servico->getQtdoaestisex(),
                        "QTCOMESTICAR" => $servico->getQtcomestisex()
                    );

                    break;

                case "SAB":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestisab(),
                        "QTFUNESTICAR" => $servico->getQtfunestisab(),
                        "QTDOAESTICAR" => $servico->getQtdoaestisab(),
                        "QTCOMESTICAR" => $servico->getQtcomestisab()
                    );

                    break;

            }

            return $estimados[0];

        }

    }

    private function retornaCardapioDoDia($cdservfili, $dtcardapio, $filialDestino) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cardadia')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "cdfilial" => $filialDestino,
                    "cdservfili" => $cdservfili,
                    "dtcardapio" => $this->retornaObjetoData($dtcardapio)
                )
            );

    }

    private function retornaComposicaoDoCardapio($nrseqpratcar, $filialDestino) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $this->nrcardapio,
                    "nrseqpratcar" => $nrseqpratcar,
                    "cdfilial" => $filialDestino
                )
            );

    }

    private function removerComposicao($cdfilial, $nrordpratcar) {

        $composicao = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $this->nrcardapio,
                    "nrordpratcar" => $nrordpratcar,
                    "cdfilial" => $cdfilial
                )
            );

        $this->entityManager->remove($composicao);
        $this->entityManager->flush();

    }

    private function retornarComposicaoDestino($cdfilial, $nrseqpratcar) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $this->nrcardapio,
                    "nrseqpratcar" => $nrseqpratcar,
                    "cdfilial" => $cdfilial
                )
            );

    }

    private function importarComposicao($montagem, $idcarddescentr, $idpratoespe, $nrseqcnfmes, $vrpeconscard, $cdprato, $cdfilauxprat, $nrordcnfmes, $cdtipoprat, $filialDestino) {
            
        $composicaoDestino = $this->retornaComposicaoDoCardapio($nrseqcnfmes, $filialDestino);
        if($composicaoDestino && $montagem == self::SOBREPOR) {
            if($idpratoespe == self::ALTERADO_FILIAL ||
                $idpratoespe == self::ALTERADO_MATRIZ &&
                $idcarddescentr) {

                $composicaoDestino->setVrpeconscard($vrpeconscard);

                if($composicaoDestino->getIdpratoespe() == self::ALTERADO_MATRIZ ||
                    $composicaoDestino->getIdpratoespe() == self::ALTERADO_FILIAL) {

                    if($composicaoDestino->getCdpratomat() == $cdprato &&
                        $composicaoDestino->getCdfilauxpratmat() == $cdfilauxprat) {

                        $composicaoDestino->setCdpratomat(null);
                        $composicaoDestino->setCdfilauxpratmat(null);
                        $composicaoDestino->setCdprato($cdprato);
                        $composicaoDestino->setCdfilauxprat($cdfilauxprat);
                        $composicaoDestino->setIdpratoespe(self::INCLUIDO_MATRIZ);

                    }else {

                        $composicaoDestino->setCdprato($cdprato);
                        $composicaoDestino->setCdfilauxprat($cdfilauxprat);
                        $composicaoDestino->setIdpratoespe($idpratoespe);

                    }

                }else if($composicaoDestino->getIdpratoespe() == self::INCLUIDO_MATRIZ) {

                    if($composicaoDestino->getCdprato() != $cdprato ||
                        $composicaoDestino->getCdfilauxprat() != $cdfilauxprat) {

                        $composicaoDestino->setCdpratomat($composicaoDestino->getCdprato());
                        $composicaoDestino->setCdfilauxpratmat($composicaoDestino->getCdfilauxprat());
                        $composicaoDestino->setCdprato($cdprato);
                        $composicaoDestino->setCdfilauxprat($cdfilauxprat);
                        $composicaoDestino->setIdpratoespe(self::ALTERADO_FILIAL);

                    }

                }

                $this->entityManager->persist($composicaoDestino);
                $this->entityManager->flush();

            }else {
                $this->entityManager->remove($composicaoDestino);
                $this->entityManager->flush();
                $this->inserirComposicao(self::INCLUIDO_MATRIZ, $nrordcnfmes, $cdtipoprat, $nrseqcnfmes, $cdfilauxprat, $cdprato, $vrpeconscard, $filialDestino);

            }

        }else if(!$composicaoDestino) {

            $this->inserirComposicao($idpratoespe, $nrordcnfmes, $cdtipoprat, $nrseqcnfmes, $cdfilauxprat, $cdprato, $vrpeconscard, $filialDestino);

        }

    }

    public function importarCardapioDoDia($idpratoespe, $idcarddescentr, $dataCardapioOrigem, $montagem, $servicoDestino, $servicoOrigem,
                                                                        $dataCardapioDestino, $diaSemanaDestino, $nrCardapioOrigem, $unidadeOrigem, $filialDestino, $idUtrPadca) {

        $cardadia = $this->retornaCardapioDoDia($servicoDestino, $dataCardapioDestino, $filialDestino);
        if(!$cardadia) {

            $composicaoOrigem = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
                ->findOneBy(
                    array(
                        "nrorg" => $this->environment->getNrOrg(),
                        "nrcardapio" => $nrCardapioOrigem,
                        "cdfilial" => $unidadeOrigem
                    )
                );

            if($composicaoOrigem) {
            
                $estimados = $this->retornaEstimados($servicoDestino, $diaSemanaDestino,$filialDestino);
                
                if($estimados['QTCOMESTICAR'] > 0)
                    $this->inserirCardapio($estimados['QTCOMESTICAR'], $estimados['QTCLIESTICAR'], $estimados['QTFUNESTICAR'],
                                                                                    $estimados['QTDOAESTICAR'], $dataCardapioDestino, $servicoDestino, $filialDestino);
                else
                    $this->nrcardapio = null;

            }
            else {
                $this->nrcardapio = null;
            }
        }
        else
            $this->nrcardapio = $cardadia->getNrcardapio();

        if(isset($this->nrcardapio)) {
            $colunasExistentes = [];
            // importar cardapio do mesmo periodo
            if((substr($dataCardapioDestino,3,7) == substr($dataCardapioOrigem,3,7)) &&
                ($unidadeOrigem == $filialDestino) &&
                ($servicoOrigem == $servicoDestino)) {
                $composicoes = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
                    ->findBy(
                        array(
                            "nrorg" => $this->environment->getNrOrg(),
                            "nrcardapio" => $nrCardapioOrigem,
                            "cdfilial" => $unidadeOrigem
                        )
                    );
                foreach ($composicoes as $composicao) {
                    array_push($colunasExistentes, $composicao->getNrseqpratcar());
                    if($idUtrPadca == 'N' && $this->tipoReceitaPadraoOuVazio($filialDestino, $servicoDestino, $composicao->getCdfilauxprat(), $composicao->getCdprato())) {
                        continue;
                    }
                    $this->importarComposicao($montagem, $idcarddescentr, $idpratoespe, $composicao->getNrseqpratcar(), $composicao->getVrpeconscard(), $composicao->getCdprato(), $composicao->getCdfilauxprat(), $composicao->getNrordpratcar(), $composicao->getCdtipoprat(), $filialDestino);
                }

            // importar cardapio periodo diferente
                
            }
            else {
                $composicoes = $this->retornaComposicoesFormatadas($nrCardapioOrigem, $unidadeOrigem, $servicoDestino, $dataCardapioDestino, $filialDestino);
                if($composicoes) {
                    foreach ($composicoes as $composicao) {
                        array_push($colunasExistentes, $composicao['nrseqcnfmes']);
                        if($idUtrPadca == 'N' && $this->tipoReceitaPadraoOuVazio($filialDestino, $servicoDestino, $composicao['cdfilauxprat'], $composicao['cdprato'])) {
                            continue;
                        }
                        $this->importarComposicao($montagem, $idcarddescentr, $idpratoespe, $composicao['nrseqcnfmes'], $composicao['vrpeconscard'], $composicao['cdprato'], $composicao['cdfilauxprat'], $composicao['nrordcnfmes'], $composicao['cdtipoprat'], $filialDestino);
                    }
                }
            }
            
            //Percorrer as colunas do cardápio e pegar as que não tem composição, e se for sobrepor, apagar a composição no destino
            //retornaComposicoesFormatadas trás o nrseq a alterar da receita de destino, por isso a rotina funciona adequadamente para os dois casos
            if($montagem == self::SOBREPOR){
                $configuracoes = $this->retornaConfiguracaoDoCardapio($servicoDestino, $dataCardapioDestino, $filialDestino);
                $tipoOperador = $this->retornaTipoOperador();
                foreach ($configuracoes as $coluna) {
                    $nrseqcnfmes = $coluna->getNrseqcnfmes();
                    //Verifica se a origem alterou a configuração corrente
                    $existeNaOrigem = array_search($nrseqcnfmes, $colunasExistentes);
                    //Se não alterou, deve apagar o prato
                    if($existeNaOrigem === false){
                        $composicaoDestino = $this->retornaComposicaoDoCardapio($nrseqcnfmes, $filialDestino);
                        if($composicaoDestino){
                            $dtmescard = '01' . substr($dataCardapioDestino,3,7);
                            //Se cardápio liberado na matriz, e serviço descentralizado, registra a compcarddel
                            if($this->cardLibPelaMatriz($filialDestino, $servicoDestino, $dtmescard)) {
                                $idlocalexcl = ($tipoOperador == 'MATRIZ') ? 'M' : 'F';
                                $cdfilauxprat = $composicaoDestino->getCdfilauxprat();
                                $cdprato = $composicaoDestino->getCdprato();
                                //inserir na COMPCARDDEL
                                $this->insereCompCardDel($filialDestino, $servicoDestino, $dataCardapioDestino, $nrseqcnfmes,
                                                         $idlocalexcl, $cdfilauxprat, $cdprato);
                            }
                            
                            $this->entityManager->remove($composicaoDestino);
                            $this->entityManager->flush();
                        }
                    }
                }
            }
        }
    }
    
    public function getIdAtivoPratosReceitasDia($row) {
        $query = "
        SELECT PT.CDPRATO, PT.NMPRATO, RECEITA_UTILIZADA(CC.CDPRATO, CC.CDFILAUXPRAT,:CDFILIAL,:CDSERVFILI) AS TIPO
            FROM CARDADIA C
            INNER JOIN COMPCARD CC ON CC.CDFILIAL = C.CDFILIAL
            AND CC.NRCARDAPIO = C.NRCARDAPIO
            INNER JOIN PRATO PT ON PT.CDFILAUXPRAT = CC.CDFILAUXPRAT AND
            PT.CDPRATO = CC.CDPRATO                              
            WHERE C.CDFILIAL = :CDFILIAL
            AND C.CDSERVFILI = :CDSERVFILI
            AND C.DTCARDAPIO = :DTCARDAPIO
            AND (
                PT.IDATIVO = 'N'
                OR EXISTS (SELECT 1 FROM PRATPADR R WHERE R.CDFILAUXPRAT = CC.CDFILAUXPRAT AND R.CDPRATO = CC.CDPRATO AND RECEITA_UTILIZADA(CC.CDPRATO, CC.CDFILAUXPRAT,:CDFILIAL_DEST,:CDSERVFILI_DEST) = 'P' AND R.IDATIVO <> 'S')
                OR EXISTS (SELECT 1 FROM PRATFILI R WHERE R.CDFILAUXPRAT = CC.CDFILAUXPRAT AND R.CDPRATO = CC.CDPRATO AND R.CDFILIAL = :CDFILIAL_DEST AND RECEITA_UTILIZADA(CC.CDPRATO, CC.CDFILAUXPRAT,:CDFILIAL_DEST,:CDSERVFILI_DEST) = 'F' AND R.IDATIVO <> 'S')
                OR EXISTS (SELECT 1 FROM PRATSERV R WHERE R.CDFILAUXPRAT = CC.CDFILAUXPRAT AND R.CDPRATO = CC.CDPRATO AND R.CDFILIAL = :CDFILIAL_DEST AND R.CDSERVFILI = :CDSERVFILI_DEST AND RECEITA_UTILIZADA(CC.CDPRATO, CC.CDFILAUXPRAT,:CDFILIAL_DEST,:CDSERVFILI_DEST) = 'S' AND R.IDATIVO <> 'S')
            )
        ";
        
        $params = array(
            "CDFILIAL"          => $row['CDFILIAL'],
            "CDSERVFILI"        => $row['CDSERVFILI'],
            "DTCARDAPIO"        => $row['DATA'],
            "CDFILIAL_DEST"     => $row['CDFILIAL_DEST'],
            "CDSERVFILI_DEST"   => $row['CDSERVFILI_DEST']
        );
        
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function tipoReceitaPadraoOuVazio($filialDestino, $servicoDestino, $cdfilauxprat, $cdprato) {
        $query = 'SELECT RECEITA_UTILIZADA(CDPRATO, CDFILAUXPRAT, :CDFILIAL, :CDSERVFILI) AS "TIPO"
          FROM PRATO
         WHERE CDFILAUXPRAT = :CDFILAUXPRAT
           AND CDPRATO = :CDPRATO'; 
           
        $params = array(
            "CDFILIAL"      => $filialDestino,
            "CDSERVFILI"    => $servicoDestino,
            "CDFILAUXPRAT"  => $cdfilauxprat,
            "CDPRATO"       => $cdprato
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($query, $params);
        
        if(count($resultado) <= 0) {
            return true;
        }
        
        $tipo = $resultado[0]["TIPO"];
        return ($tipo == null || $tipo == 'P');
        
    }
    
    private function insereCompCardDel($cdfilial, $cdservfili, $dtpratexcl, $nrseqpratexcl, $idlocalexcl, $cdfilauxprat, $cdprato){
        $sql = "
        INSERT INTO COMPCARDDEL
        (CDFILIAL, CDSERVFILI, DTPRATEXCL, 
        NRSEQPRATEXCL, IDLOCALEXCL, CDFILAUXPRAT, CDPRATO)
        SELECT 
        :P_CDFILIAL, :P_CDSERVFILI, :P_DTPRATEXCL,
        :P_NRSEQPRATEXCL, :P_IDLOCALEXCL,
        :P_CDFILAUXPRAT, :P_CDPRATO
        FROM DUAL
        WHERE NOT EXISTS 
        (SELECT 1
        FROM COMPCARDDEL
        WHERE CDFILIAL = :P_CDFILIAL
        AND CDSERVFILI = :P_CDSERVFILI
        AND DTPRATEXCL = :P_DTPRATEXCL
        AND NRSEQPRATEXCL = :P_NRSEQPRATEXCL)
        ";
        
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_CDSERVFILI" => $cdservfili,
            "P_DTPRATEXCL" => $dtpratexcl,
            "P_NRSEQPRATEXCL" => $nrseqpratexcl,
            "P_IDLOCALEXCL" => $idlocalexcl,
            "P_CDFILAUXPRAT" => $cdfilauxprat,
            "P_CDPRATO" => $cdprato
        );
        
        $cmd = $this->entityManager->getConnection()->prepare($sql);
        $cmd->execute($params);
    }
    
    /**
     * Método que retorna o tipo de operador, sendo ele MATRIZ ou FILIAL.
     *
     * @return String
     */
    public function retornaTipoOperador() {
        $row = array(
            'CDOPERADOR'   => $this->environment->getCdOperador(),
            'AUTENTICACAO' => array(
    			'COLUMN_NAME'=> null,
    			'ALIAS_NAME'=> 'LRCD'
		    )
        );
        $permission = $this->auth->returnOperatorInGroup($row);

        return $permission ? 'MATRIZ' : 'FILIAL';

    }
    
    private function cardLibPelaMatriz($cdfilial, $cdservfili, $dtmescard) {
        $query = "SELECT 1
        FROM CTRLAPROVCARD CTRL
        WHERE
        CDFILIAL = :P_CDFILIAL AND
        CDSERVFILI = :P_CDSERVFILI AND
        DTMESCARDAPROV = :P_DTMESCALENCARD AND
        DTLIBECARDMATR IS NOT NULL";
        
        $params = [
            "P_CDFILIAL" => $cdfilial,
            "P_CDSERVFILI" => $cdservfili,
            "P_DTMESCALENCARD" => $dtmescard
        ];
        
        $sql = $this->entityManager->getConnection()->prepare($query);
        
        $resultado = $this->entityManager->getConnection()->fetchAll($query, $params);
        
        if (count($resultado) > 0){
            return true;
        }
        
        return false;
    }
    
    public function verificaFeriado($cdFilial, $dtFeriFili) {
        $query = "SELECT DTFERIFILI
        FROM FERIFILI
        WHERE CDFILIAL = :P_CDFILIAL
        AND DTFERIFILI = :P_DTFERIFILI";
        
        $params = [
            "P_CDFILIAL"    => $cdFilial,
            "P_DTFERIFILI"  => $dtFeriFili
        ];
        
        $sql = $this->entityManager->getConnection()->prepare($query);
        $resultado = $this->entityManager->getConnection()->fetchAll($query, $params);    
        return count($resultado) > 0;
    }

}