<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;

use Teknisa\GER\Modules\Cardapio\Controllers\TeknisaCrud;
use Teknisa\GER\Utilities\Environment;
use Doctrine\ORM\EntityManager;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Pla06600SubstPratoCardaService {

    protected $parameterBagColumns = array();
    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $novoCodigo;

    const EXISTENCIA_CARDAPIOS = 'pla06600_existecardapio';
    const PARAMGERAL = 'paramgeral';
    const VALIDA_DATA_PROC = 'pla06600_validadataproc';
    const VALIDA_GRUP_OPERADOR = 'pla06600_validagrupoperador';
    const VALIDA_LIB_CARDAPIOS = 'pla06600_validalibcardapios';

    public function __construct(Environment $environment, $entityManager, $dataSourceManager, $novoCodigo) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->novoCodigo = $novoCodigo;
    }

    public function novoCodigoLista() {
        $contador = 'LISTA';
        $nrorg = $this->environment->getNrOrg();
        $nrlista = $this->novoCodigo->getCodeByTableName($contador, $nrorg);
        return $nrlista;
    }

    public function insereLista($cdlista, $nrlista) {
        $lista = new \Teknisa\Model\Geral\Entities\Lista();
        $lista = $this->constroiLista($lista, $cdlista, $nrlista);
        $this->entityManager->persist($lista);
        $this->entityManager->flush();
    }

    public function constroiLista($lista, $cdlista, $nrlista) {
        $lista->setCdlista($cdlista);
        $lista->setNrlista($nrlista);
        $lista->setNrorg($this->environment->getNrOrg());
        $lista->setCdoperinclusao($this->environment->getCdOperador());
        $lista->setDtinclusao(new DateTime());
        $lista->setDtultatu(new DateTime());
        $lista->setIdativo('S');
        return $lista;
    }

    public function excluiLista($nrlista) {
        $this->entityManager->getConnection()->beginTransaction();
        $listas = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Lista')->findBy(
            array("nrlista" => $nrlista, "nrorg" => $this->environment->getNrOrg())
        );
        foreach ($listas as $lista) {
            $this->entityManager->remove($lista);
            $this->entityManager->flush();
        }
        $this->entityManager->getConnection()->commit();
    }

    public function excluiListaErros($nrlista) {
        $sql    = "DELETE FROM LISTAERROS WHERE NRLISTA = :P_NRLISTA AND NRORG = :P_NRORG";
        $params = array(
            "P_NRLISTA" => $nrlista,
            "P_NRORG"   => $this->environment->getNrOrg()
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }

    public function verificaParamgeral($cdpargeral, $nrorg) {
        $filterCriteria = new FilterCriteria(self::PARAMGERAL);
        $filterCriteria->addCondition('CDPARGERAL', $cdpargeral);
        $filterCriteria->addCondition('NRORG', $nrorg);
        return $filterCriteria;
    }

    public function verificaExistenciaCardapios($listaserv, $listafili, $dtinicial, $dtfinal, $cdprato, $cdfilaux, $nrorg) {
        $filterCriteria = new FilterCriteria(self::EXISTENCIA_CARDAPIOS);
        $filterCriteria->addCondition('LISTASERV', $listaserv);
        $filterCriteria->addCondition('LISTAFILI', $listafili);
        $filterCriteria->addCondition('DTINICIAL', $dtinicial);
        $filterCriteria->addCondition('DTFINAL', $dtfinal);
        $filterCriteria->addCondition('CDPRATO', $cdprato);
        $filterCriteria->addCondition('CDFILAUX', $cdfilaux);
        $filterCriteria->addCondition('NRORG', $nrorg);
        return $filterCriteria;
    }

    public function verificaDataProc($listafili, $cdpargeral, $dtinicial, $nrorg) {
        $filterCriteria = new FilterCriteria(self::VALIDA_DATA_PROC);
        $filterCriteria->addCondition('NRLISTA', $listafili);
        $filterCriteria->addCondition('CDPARGERAL', $cdpargeral);
        $filterCriteria->addCondition('DTINICIAL', $dtinicial);
        $filterCriteria->addCondition('NRORG', $nrorg);
        return $filterCriteria;
    }

    public function verificaGrupOperador($cdoperador, $nrorg) {
        $filterCriteria = new FilterCriteria(self::VALIDA_GRUP_OPERADOR);
        $filterCriteria->addCondition('CDOPERADOR', $cdoperador);
        $filterCriteria->addCondition('NRORG', $nrorg);
        return $filterCriteria;
    }

    public function verificaLibCarda($cdfilial, $cdservfili, $dtmescard, $nrorg) {
        $filterCriteria = new FilterCriteria(self::VALIDA_LIB_CARDAPIOS);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTMESCARD', $dtmescard);
        $filterCriteria->addCondition('NRORG', $nrorg);
        return $filterCriteria;
    }

    public function TROCA_PRATO_CARD($listafili, $listaserv, $dtinicial, $dtfinal, $cdpratorig, $cdfapratorig, $cdpratdest, $cdfapratdest, $modoproc, $cdnvproduto, $cdoperador, $nrlogintcard) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'TROCA_PRATO_CARD');
        $procedure->addParam(new Param('P_LISTAFILI', Param::PARAM_INPUT, $listafili, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTASERV', Param::PARAM_INPUT, $listaserv, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRATORIG', Param::PARAM_INPUT, $cdpratorig, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFAPRATORIG', Param::PARAM_INPUT, $cdfapratorig, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRATDEST', Param::PARAM_INPUT, $cdpratdest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFAPRATDEST', Param::PARAM_INPUT, $cdfapratdest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_MODOPROC', Param::PARAM_INPUT, $modoproc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDNVPRODUTO', Param::PARAM_INPUT, $cdnvproduto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOGINTCARD', Param::PARAM_OUTPUT, $nrlogintcard, Param::PARAM_TYPE_STR, 10));
        $nrlogintcard = $procedure->execute();
        return $nrlogintcard;
    }
    
    public function TROCA_CONSUMO_PRATO_CARD($listaServ, $dtInicial, $dtFinal, $cdPrato, $cdFilAuxPrat, $consumo, $valor, $cdOperador, $nrLogIntCard){
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'TROCA_CONSUMOPRATO_CARD');
        $procedure->addParam(new Param('P_LISTASERV', Param::PARAM_INPUT, $listaServ, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $dtInicial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $dtFinal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRATO', Param::PARAM_INPUT, $cdPrato, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILAUXPRAT', Param::PARAM_INPUT, $cdFilAuxPrat, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONSUMO', Param::PARAM_INPUT, $consumo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VALOR', Param::PARAM_INPUT, $valor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdOperador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOGINTCARD', Param::PARAM_OUTPUT, $nrLogIntCard, Param::PARAM_TYPE_STR, 10));
        $nrLogIntCard = $procedure->execute();
        return $nrLogIntCard;
    }
    
    public function checkActiveRecipe($row, $nrlistaservfili, $type){
        $sql    = "SELECT S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO, :P_TIPO AS TIPO FROM ( SELECT S.CDSERVFILI, S.CDFILIAL, S.NMSERVICO, PRATO_RECEITA_ATIVA(:P_CDPRATO, :P_CDFILAUX, S.CDFILIAL, S.CDSERVFILI, :P_TIPO) AS IDATIVO FROM SERVICO S, LISTA L WHERE L.NRLISTA = :P_LISTASERV AND S.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4) AND S.CDSERVFILI = SUBSTR(L.CDLISTA, 5, 5) ) S WHERE S.IDATIVO = 'N' ORDER BY S.CDFILIAL, S.CDSERVFILI";
        $params = array(
            "P_LISTASERV" => $nrlistaservfili,
            "P_CDPRATO"   => $row[0]['CDPRATOSUBST'],
            "P_CDFILAUX"  => $row[0]['CDFILAUXPRATSUBST'],
            "P_TIPO"      => $type
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
        public function listaErros($nrLista){
        $sql    = "SELECT DSERRO FROM LISTAERROS WHERE NRLISTA = :P_NRLISTA AND NRORG = :P_NRORG";
        $params = array(
            "P_NRLISTA" => $nrLista,
            "P_NRORG"   => $this->environment->getNrOrg()
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

}