<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;

use Teknisa\GER\Modules\Cardapio\Controllers\TeknisaCrud;
use Teknisa\GER\Utilities\Environment;
use Doctrine\ORM\EntityManager;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Pla06900LiberacaoDeCardapioParaProducaoService {
    const GET_ALL_SERVICES = "pla06800_getServico";
    
    public function __construct(Environment $environment, $entityManager, $dataSourceManager, $novoCodigo) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->novoCodigo = $novoCodigo;
    }
    
    public function getAllAgregadas($central) {
        $sql    = "SELECT CDFILIAL, NMFILIAL, NRORG FROM FILIAL F WHERE F.CDFILCENTPME = :P_CDFILIAL AND EXISTS (SELECT 1 FROM FILIOPER FO WHERE FO.CDFILIAL = F.CDFILIAL AND FO.CDOPERADOR = :CDOPERADOR) ORDER BY CDFILIAL";
        
        $params = array(
        "CDOPERADOR"    => $this->environment->getCdOperador(),
        "P_CDFILIAL"    => $central
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function getAllServices($central, $agregada) {
        $sql    = "SELECT F.CDFILIAL, S.CDSERVFILI, F.NMFILIAL, S.CDFILIAL || S.CDSERVFILI AS CDCHAVE, S.NMSERVICO, S.NRORG FROM FILIAL F, SERVICO S WHERE F.CDFILCENTPME = :P_CDCENTRAL AND (INSTR(:P_CDFILIAL, F.CDFILIAL) <> 0 OR :P_CDFILIAL = 'T') AND F.CDFILIAL = S.CDFILIAL AND IDTIPOSERV = 'N' ORDER BY S.CDFILIAL, S.CDSERVFILI";
        $params = array(
            "P_CDCENTRAL"   => $central,
            "P_CDFILIAL"    => $agregada
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }  
    
    public function updateCardaMes($cdFilial, $cdServFili, $dtMes) {
        $sql = "UPDATE CARDAMES SET IDCARDLIBAGRE = 'N' WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTMESCARDAPIO = :P_DTMES";
        $paramsUpdate = [
            "P_CDFILIAL"    => $cdFilial,
            "P_CDSERVFILI"  => $cdServFili,
            "P_DTMES"       => $dtMes
        ];  
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($paramsUpdate);     
    }
    
    public function deleteCtrlAprovCard($cdFilial, $cdServFili, $dtMes) {
        $sql = "DELETE FROM CTRLAPROVCARD WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTMESCARDAPROV = :P_DTMESCARD";
        $paramsDelete = [
            "P_CDFILIAL"    => $cdFilial,
            "P_CDSERVFILI"  => $cdServFili,
            "P_DTMESCARD"   => $dtMes
        ];        
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($paramsDelete);   
    }
    
    public function executaConsplanGeral($row, $nrLista) {
        $cdOperador = $this->environment->getCdOperador();
        $consplanGeral = new StoredProcedure($this->entityManager->getConnection(), 'CONSPLAN_GERAL');

        $consplanGeral->addParam(new Param('P_NRLISTA',      Param::PARAM_INPUT, $nrLista,                          Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['FORMATEDDTCARDAPIO'],        Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['FORMATEDDTCARDAPIOEND'],     Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, 'CFP',                             Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'N',                               Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                                 Param::PARAM_TYPE_INT));
        $consplanGeral->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, 'N',                               Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, '0',                               Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, 'zzzzzzzzzzzzzzzzzzz',             Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdOperador,                       Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_NRCONSPLANTH', Param::PARAM_INPUT, 'GERANOVCOD',                      Param::PARAM_TYPE_STR));
        $consplanGeral->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_OUTPUT, null,                             Param::PARAM_TYPE_STR, 20));

        return $consplanGeral->execute();        
    }
    
    public function retornaMediaEMeta($row, $servico) {
        $sql = "SELECT CP.CDFILICPLA, CP.CDSERVCPLA, C.VRCUSTPADR AS META, CP.MEDIA FROM (SELECT CDFILICPLA, CDSERVCPLA, CD.CDFILIAL, CD.CDSERVFILI, CD.DTREFECUSTMES, CD.TOTCOMENS, SUM(DECODE(CD.TOTCOMENS, 0, 0, NULL, 0, (CUSTO_PRODUTO(CDFILICPLA, CDPRODCPLA, :P_TIPOCUSTO, CD.DTREFECUSTMES, SYSDATE) * QTCONSPLAN) / CD.TOTCOMENS)) AS MEDIA FROM CONSPLAN, (SELECT C.CDFILIAL, C.CDSERVFILI, NVL(CM.DTREFECUSTMES, SYSDATE) AS DTREFECUSTMES, SUM(DECODE(P.IDCOMCUSPERC, 'T', C.QTCOMESTICAR, C.QTCLIESTICAR)) AS TOTCOMENS FROM CARDADIA C, PARAMFILIAL P, CARDAMES CM WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND C.DTMESCARD = :P_DTMES AND C.CDFILIAL = P.CDFILIAL AND C.CDFILIAL = CM.CDFILIAL AND C.CDSERVFILI = CM.CDSERVFILI AND C.DTMESCARD = CM.DTMESCARDAPIO GROUP BY C.CDFILIAL, C.CDSERVFILI, CM.DTREFECUSTMES) CD WHERE NRCONSPLAN = :P_NRCONSPLAN AND CDFILICPLA = :P_CDFILIAL AND CDFILICPLA = CD.CDFILIAL AND CDSERVCPLA = CD.CDSERVFILI GROUP BY CDFILICPLA, CDSERVCPLA, CD.CDFILIAL, CD.CDSERVFILI, CD.DTREFECUSTMES, CD.TOTCOMENS) CP, CUSTPADR C WHERE C.DTREFCUSPADR(+) = :P_DTMES AND CP.CDFILICPLA = C.CDFILIAL(+) AND CP.CDSERVCPLA = C.CDSERVFILI(+)";
        $params = array(
            "P_CDFILIAL"    => $servico["CDFILIAL"],
            "P_TIPOCUSTO"   => $row["CDTIPOCUSTO"],
            "P_CDSERVFILI"  => $servico["CDSERVFILI"],
            "P_NRCONSPLAN"  => $row["NRCONSPLAN"]["P_NRCONSPLAN"],
            "P_DTMES"       => $row["FORMATEDDTCARDAPIO"]
        );    
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();          
    }
    
    public function retornCustPadr($row, $servico) {
        $sql = "SELECT VRCUSTPADR FROM CUSTPADR WHERE CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTREFCUSPADR = :P_DTREFCUSPADR";
        $params = [
            "P_CDFILIAL"        => $servico["CDFILIAL"],
            "P_CDSERVFILI"      => $servico["CDSERVFILI"],
            "P_DTREFCUSPADR"    => $row["FORMATEDDTCARDAPIO"]
        ];
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);
        return $query->fetchAll();         
    } 
    
    public function verificaServicoLiberado($row, $servico) {
        $sql = "SELECT CDFILIAL, CDSERVFILI, IDCARDLIBCENT, IDCARDLIBAGRE FROM CARDAMES WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTMESCARDAPIO = :P_DTMESCARD";
        $params = [
            "P_CDFILIAL"        => $servico["CDFILIAL"],
            "P_CDSERVFILI"      => $servico["CDSERVFILI"],
            "P_DTMESCARD"       => $row["FORMATEDDTCARDAPIO"]
        ];
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);
        return $query->fetchAll(); 
    }
    
    public function updateCardaMesLibCardProd($cdFilial, $cdServFili, $dtMes) {
        $sql = "UPDATE CARDAMES SET IDCARDLIBAGRE = 'S', DTREFECUSTMES = TRUNC(SYSDATE) WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTMESCARDAPIO = :P_DTMES";
        $paramsUpdate = [
            "P_CDFILIAL"    => $cdFilial,
            "P_CDSERVFILI"  => $cdServFili,
            "P_DTMES"       => $dtMes
        ];  
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($paramsUpdate);        
    }
    
    public function verificaVrCustoLib($cdFilial, $cdServFili, $dtMes) {
        $sql = "SELECT DTAPROCARDFILI, VRCUSTOLIB FROM CTRLAPROVCARD WHERE (CDFILIAL = :P_CDFILIAL) AND (CDSERVFILI = :P_CDSERVFILI) AND (DTMESCARDAPROV = :P_DTMESCARD)";
        $params = [
            "P_CDFILIAL"        => $cdFilial,
            "P_CDSERVFILI"      => $cdServFili,
            "P_DTMESCARD"       => $dtMes
        ];
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);
        return $query->fetchAll(); 
    }
    
    public function updateCtrlAprovCard($row, $servico) {
        $sql = "UPDATE CTRLAPROVCARD SET DTAPROCARDFILI = SYSDATE, CDOPERAPROCARDFILI = :P_CDOPERADOR, VRCUSTOLIB = :P_VRCUSTOLIB WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTMESCARDAPROV = :P_DTMESCARD";
        $paramsUpdate = [
            "P_CDOPERADOR"  => $this->environment->getCdOperador(),
            "P_VRCUSTOLIB"  => $servico["VRCUSTO"],
            "P_CDFILIAL"    => $servico["CDFILIAL"],
            "P_CDSERVFILI"  => $servico["CDSERVFILI"],
            "P_DTMESCARD"   => $row["FORMATEDDTCARDAPIO"]
        ];  
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($paramsUpdate);        
    }
    
    public function insertCtrAprovCard($row, $servico) {
        $sql = "INSERT INTO CTRLAPROVCARD (CDFILIAL, CDSERVFILI, DTMESCARDAPROV, DTAPROCARDFILI, CDOPERAPROCARDFILI, VRCUSTOLIB ) VALUES (:P_CDFILIAL, :P_CDSERVFILI, :P_DTMESCARD, SYSDATE, :P_CDOPERADOR, :P_VRCUSTOLIB )";
        $paramsInsert = [
            "P_CDFILIAL"    => $servico["CDFILIAL"],
            "P_CDSERVFILI"  => $servico["CDSERVFILI"],
            "P_DTMESCARD"   => $row["FORMATEDDTCARDAPIO"],
            "P_CDOPERADOR"  => $this->environment->getCdOperador(),
            "P_VRCUSTOLIB"  => $servico["VRCUSTO"]
        ];        
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($paramsInsert);   
    }    
    
}