<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\GER\Utilities\Environment;

class Pla11700IncidenciaDeCardapio extends \Zeedhi\Framework\Controller\Report {
      
    
	protected $environment;
	protected $dataSourceManager;
	protected $entityManager;
	protected $birtStrategy;
	protected $crudService;
	protected $newCode;
        
    public function __construct($reportService, $environment, $dataSourceManager, $entityManager, $birtStrategy, $crudService, $newCode) {
		$this->reportService = $reportService;
		$this->birtStrategy = $birtStrategy;
        $this->environment = $environment;
		$this->dataSourceManager = $dataSourceManager;
		$this->entityManager = $entityManager;
		$this->crudService = $crudService;
		$this->newCode = $newCode;
	}
	
	protected $reportMapping = array (
		'PLA11700' =>
		array (
			'strategy' => 'BIRT',
			'parameterMapping' =>
			array (
				"CDGRUPFILI"     => "P_CDGRUPFILI",
                "CDGRUPSERV"     => "P_CDGRUPSERV",
                "P_NRLISTASERV"  => "P_NRLISTASERV",
                "DTINI" 	     => "P_DTINI",
                "DTFIN"          => "P_DTFIN",
                "P_CDPRATINI"    => "P_CDPRATOINI",
                "TIPONIVEL"      => "P_CDNVPRATO",
                "P_CDPRATFIN"    => "P_CDPRATOFIN",
                "IDIOMA"         => "P_IDIOMA",
                "P_NRORG"        => "P_NRORG",
                "NMTIPOPRAT"     => "P_NMTIPOPRAT"
			)
		)
	);
	
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		if($row['P_CDPRATINI'] == ""){
			$row['P_CDPRATINI'] = "0";
			$row['P_CDPRATFIN'] = "z";
		}
		
		if (is_array($row['NMTIPOPRAT'])) {
            $valorTipoPrato = implode('_', $row['NMTIPOPRAT']);
            $row['NMTIPOPRAT'] = $valorTipoPrato;
        }
		
		if($row['CDFILIAL'] == 'T') $row['CDFILIAL'] = $this->geraListaFilial($row);
		if($row['CDSERVFILI'] == 'T') $row['CDSERVFILI'] = $this->geraListaServ($row);
		
		$row['P_NRLISTASERV'] = $this->insertLista($row['CDSERVFILI']);
		$row['P_NRORG'] = $this->environment->getNrOrgTrab();
		
		$this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
	}

	private function geraListaFilial($row){
		$grupfili = $row['CDGRUPFILI'] ? $row['CDGRUPFILI'] : 'T';
		$conditions = array(
            array(
                'columnName' => 'CDGRUPFILI',
                'operator' => '=',
                'value' => $grupfili
            ),
            array(
                'columnName' => 'CDOPERADOR',
                'operator' => '=',
                'value' => $this->environment->getCdOperador()
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrg()
            ),
            array(
               'columnName' => 'IDCNTRRESC',
                'operator' => '=',
                'value' => 'N'
            )
        );
        
        $filterCriteria = new FilterCriteria('filial_operador_grupfili', $conditions);
        $paramBag = array("NRORG", "CDOPERADOR", "CDGRUPFILI");
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $paramBag);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $rows = $dataSet->getRows();

        $concatFiliais = '_';
        foreach($rows as $filial){
            $concatFiliais = $concatFiliais.$filial['CDFILIAL'].'_';
		}
        return $concatFiliais;
	}

	private function geraListaServ($row){
		$grupfili = $row['CDGRUPFILI'] ? $row['CDGRUPFILI'] : 'T';
		$conditions = array(
            array(
                'columnName' => 'CDGRUPFILI',
                'operator' => '=',
                'value' => $grupfili
            ),
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'CDGRUPSERV',
                'operator' => '=',
                'value' => $row['CDGRUPSERV']
            ),
            array(
                'columnName' => 'CDOPERADOR',
                'operator' => '=',
                'value' => $this->environment->getCdOperador()
            ),
            array(
               'columnName' => 'IDCNTRRESC',
                'operator' => '=',
                'value' => 'N'
            )
        );
        $filterCriteria = new FilterCriteria('servico_operador_filgrupfili_filial_grupserv', $conditions);
        $paramBag = array("CDGRUPFILI", "CDFILIAL", "CDGRUPSERV", "CDOPERADOR");
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $paramBag);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $rows = $dataSet->getRows();

        $concatServ = '_';
        foreach($rows as $filial){
            $concatServ = $concatServ.$filial['CHAVE'].'_';
		}
        return $concatServ;
	}

	private function insertLista($array) {
        $nrlista = $this->newCode->getCodeByTableName("LISTA", $this->environment->getNrOrg());
		if(gettype($array) != 'array') $array = explode("_", $array);
		$data = [];
		foreach ($array as $reg) {
			if($reg){
				$row = array(
					"NRLISTA" => $nrlista,
					"CDLISTA" => $reg,
					"__is_new" => true
				);
				array_push($data, $row);
			}
            
        }
        $dataSet = new DataSet("lista", $data);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, "lista");
        $this->crudService->save("lista", $rowsToSave);
        return '_NRLISTA='.$nrlista;
    }
}