<?php

namespace Teknisa\PLA\Modules\Servico\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla04105ServicosPorRota {
	
	protected $connection;
	protected $environment;

    public function __construct($connection, $environment){
        $this->connection    = $connection;
        $this->environment   = $environment;
	}
	
    public function associar ($request, $response){
        $rows = $request->getRow();
        $codigoRota = $rows['CDROTA'];
        
        if($rows['SERVICOS'] == 'T'){ 
            $unidades = $rows['UNIDADES'];    
            $this->realizarUpdatePorUnidade($unidades, $codigoRota); //Todos os serviços da(s) unidade(s) selecionada(s)
        }else{
            $servicos = $rows['SERVICOS'];
            $this->realizarUpdatePorServico($servicos, $codigoRota); //Realizar update em um serviço especifico que foi selecionado no front. O serviço já possui seu codigo e filial
        }
    }
    
    private function realizarUpdatePorServico($servicos, $codigoRota){
        foreach ($servicos as $servico) {
            try{
        	    $servicoQuery = "UPDATE SERVICO
                               SET CDROTATRANS = :CDROTATRANS
                             WHERE CDFILIAL = :CDFILIAL
                               AND CDSERVFILI = :CDSERVFILI";
                               
                $stmt = $this->connection->prepare($servicoQuery);
                $params = array(
                    "CDROTATRANS"    => $codigoRota,
                    "CDFILIAL"       => $servico['CDFILIAL'],
                    "CDSERVFILI"     => $servico["CDSERVFILI"]
                );
                $stmt->execute($params);
            }catch(\Exception $e){
                throw new \Exception($e->getMessage());
            }
        }
    }
    
    private function realizarUpdatePorUnidade($unidades, $codigoRota){
        foreach ($unidades as $unidade) {
            try{
        	    $unidadeQuery = "UPDATE SERVICO
                               SET CDROTATRANS = :CDROTATRANS
                             WHERE CDFILIAL = :CDFILIAL";
                               
                $stmt = $this->connection->prepare($unidadeQuery);
                $params = array(
                    "CDROTATRANS"    => $codigoRota,
                    "CDFILIAL"       => $unidade['CDFILIAL']
                );
                $stmt->execute($params);
            }catch(\Exception $e){
                throw new \Exception($e->getMessage());
            }
        }
    }
    
    public function excluir($request, $response){
         $rows = $request->getRow();
         $servicos = $rows['SERVICOS'];
         foreach($servicos as $servico){
             try{
        	    $servicoQuery = "UPDATE SERVICO
                                   SET CDROTATRANS = NULL
                                 WHERE CDFILIAL = :CDFILIAL
                                   AND CDSERVFILI = :CDSERVFILI";
                               
                $stmt = $this->connection->prepare($servicoQuery);
                $params = array(
                    "CDFILIAL"   => $servico['CDFILIAL'],
                    "CDSERVFILI" => $servico['CDSERVFILI']
                );
                $stmt->execute($params);
            }catch(\Exception $e){
                throw new \Exception($e->getMessage());
            }
        }
    }

}